@@ -26,126 +26,25 @@
 #include <net/address.h>
 #include "key.h"
 #include "rpccall.h"
-
-using bt::Uint32;
-using bt::Uint16;
-using bt::Uint8;
+#include "kbucketentry.h"
 
 namespace bt
 {
 	class File;
+	class BEncoder;
 }
 
 namespace dht
 {
-	class RPCServer;
+	class RPCServerInterface;
 	class KClosestNodesSearch;
-	class Node;
 	class Task;
-	
-	const Uint32 K = 20;
-	const Uint32 BUCKET_MAGIC_NUMBER = 0xB0C4B0C4;
-	const Uint32 BUCKET_REFRESH_INTERVAL = 15 * 60 * 1000;
-//	const Uint32 BUCKET_REFRESH_INTERVAL = 120 * 1000;
-	
-	struct BucketHeader
-	{
-		Uint32 magic;
-		Uint32 index;
-		Uint32 num_entries;
-	};
-	
-	/**
-	 * @author Joris Guisson
-	 *
-	 * Entry in a KBucket, it basically contains an ip_address of a node,
-	 * the udp port of the node and a node_id.
-	 */
-	class KBucketEntry
-	{
-		net::Address addr;
-		Key node_id;
-		bt::TimeStamp last_responded;
-		Uint32 failed_queries;
-		Uint32 questionable_pings;
-	public:
-		/**
-		 * Constructor, sets everything to 0.
-		 * @return 
-		 */
-		KBucketEntry();
-		
-		/**
-		 * Constructor, set the ip, port and key
-		 * @param addr socket address
-		 * @param id ID of node
-		 */
-		KBucketEntry(const net::Address & addr,const Key & id);
-		
-		/**
-		 * Copy constructor.
-		 * @param other KBucketEntry to copy
-		 * @return 
-		 */
-		KBucketEntry(const KBucketEntry & other);
 
-		/// Destructor
-		virtual ~KBucketEntry();
+	const bt::Uint32 K = 20;
+	const bt::Uint32 BUCKET_MAGIC_NUMBER = 0xB0C4B0C4;
+	const bt::Uint32 BUCKET_REFRESH_INTERVAL = 15 * 60 * 1000;
 
-		/**
-		 * Assignment operator.
-		 * @param other Node to copy
-		 * @return this KBucketEntry
-		 */
-		KBucketEntry & operator = (const KBucketEntry & other);
-		
-		/// Equality operator
-		bool operator == (const KBucketEntry & entry) const;
-		
-		/// Get the socket address of the node
-		const net::Address & getAddress() const {return addr;}
-		
-		/// Get it's ID
-		const Key & getID() const {return node_id;}
-		
-		/// Is this node a good node
-		bool isGood() const;
-		
-		/// Is this node questionable (haven't heard from it in the last 15 minutes)
-		bool isQuestionable() const;
-		
-		/// Is it a bad node. (Hasn't responded to a query
-		bool isBad() const;
-		
-		/// Signal the entry that the peer has responded
-		void hasResponded();
-		
-		/// A request timed out
-		void requestTimeout() {failed_queries++;}
 
-		/// The entry has been pinged because it is questionable
-		void onPingQuestionable() {questionable_pings++;}
-		
-		/// The null entry
-		static KBucketEntry null; 
-		
-		/// < operator
-		bool operator < (const KBucketEntry & entry) const;
-	};
-	
-	class KBucketEntrySet : public std::set<KBucketEntry>
-	{
-	public:
-		KBucketEntrySet() {}
-		virtual ~KBucketEntrySet() {}
-		
-		bool contains(const KBucketEntry & entry) const 
-		{
-			return find(entry) != end();
-		}
-	};
-	
-	
 	/**
 	 * @author Joris Guisson
 	 *
@@ -157,69 +56,97 @@ namespace dht
 	class KBucket : public RPCCallListener
 	{
 		Q_OBJECT
-				
-		Uint32 idx;
-		QList<KBucketEntry> entries,pending_entries;
-		RPCServer* srv;
-		Node* node;
-		QMap<RPCCall*,KBucketEntry> pending_entries_busy_pinging;
-		mutable bt::TimeStamp last_modified;
-		Task* refresh_task;
+
 	public:
-		KBucket(Uint32 idx,RPCServer* srv,Node* node);
+		KBucket(RPCServerInterface* srv, const Key & our_id);
+		KBucket(const dht::Key & min_key, const dht::Key & max_key, RPCServerInterface* srv, const Key & our_id);
 		virtual ~KBucket();
 		
-		/**
-		 * Inserts an entry into the bucket. 
-		 * @param entry The entry to insert
-		 */
-		void insert(const KBucketEntry & entry);
+		typedef QSharedPointer<KBucket> Ptr;
 		
+		/// Get the min key
+		const dht::Key & minKey() const {return min_key;}
+		
+		/// Get the max key
+		const dht::Key & maxKey() const {return max_key;}
+
+		/// Does the key k lies in in the range of this bucket
+		bool keyInRange(const dht::Key & k) const;
+		
+		/// Are we allowed to split
+		bool splitAllowed() const;
+		
+		class UnableToSplit
+		{};
+		
+		/**
+		 * Split the bucket in two new buckets, each containing half the range of the original one.
+		 * @return A pair of KBucket's
+		 * @throw UnableToSplit if something goes wrong
+		 */
+		std::pair<KBucket::Ptr, KBucket::Ptr> split() throw (UnableToSplit);
+		
+		/**
+		 * Inserts an entry into the bucket.
+		 * @param entry The entry to insert
+		 * @return true If the bucket needs to be splitted, false otherwise
+		 */
+		bool insert(const KBucketEntry & entry);
+
 		/// Get the least recently seen node
 		const KBucketEntry & leastRecentlySeen() const {return entries[0];}
-		
+
 		/// Get the number of entries
-		Uint32 getNumEntries() const {return entries.count();}
-	
+		bt::Uint32 getNumEntries() const {return entries.count();}
+
 		/// See if this bucket contains an entry
 		bool contains(const KBucketEntry & entry) const;
-		
+
 		/**
 		 * Find the K closest entries to a key and store them in the KClosestNodesSearch
 		 * object.
 		 * @param kns The object to storre the search results
 		 */
 		void findKClosestNodes(KClosestNodesSearch & kns);
-		
+
 		/**
 		 * A peer failed to respond
 		 * @param addr Address of the peer
 		 */
 		bool onTimeout(const net::Address & addr);
-		
+
 		/// Check if the bucket needs to be refreshed
 		bool needsToBeRefreshed() const;
-		
+
 		/// save the bucket to a file
-		void save(bt::File & fptr);
-		
+		void save(bt::BEncoder & enc);
+
 		/// Load the bucket from a file
-		void load(bt::File & fptr,const BucketHeader & hdr);
-		
+		void load(bt::BDictNode* dict);
+
 		/// Update the refresh timer of the bucket
 		void updateRefreshTimer();
-		
+
 		/// Set the refresh task
 		void setRefreshTask(Task* t);
-		
+
 	private:
-		virtual void onResponse(RPCCall* c,MsgBase::Ptr rsp);
+		virtual void onResponse(RPCCall* c, RPCMsg::Ptr rsp);
 		virtual void onTimeout(RPCCall* c);
 		void pingQuestionable(const KBucketEntry & replacement_entry);
 		bool replaceBadEntry(const KBucketEntry & entry);
-		
+
 	private slots:
 		void onFinished(Task* t);
+		
+	private:
+		dht::Key min_key, max_key;
+		QList<KBucketEntry> entries, pending_entries;
+		RPCServerInterface* srv;
+		Key our_id;
+		QMap<RPCCall*, KBucketEntry> pending_entries_busy_pinging;
+		mutable bt::TimeStamp last_modified;
+		Task* refresh_task;
 	};
 }
 
