@@ -37,7 +37,7 @@
 #include <net/address.h>
 #include <torrent/torrent.h>
 #include <util/functions.h>
-#include <mse/encryptedpacketsocket.h> 
+#include <mse/encryptedpacketsocket.h>
 #include <mse/encryptedauthenticate.h>
 #include <peer/accessmanager.h>
 #include <torrent/globals.h>
@@ -48,55 +48,52 @@
 #include "peer.h"
 #include "authenticate.h"
 #include "peerconnector.h"
+#include "connectionlimit.h"
 
 
 using namespace KNetwork;
 
 namespace bt
 {
-	Uint32 PeerManager::max_connections = 0;
-	Uint32 PeerManager::max_total_connections = 0;
-	Uint32 PeerManager::total_connections = 0;
-	
 	typedef std::map<net::Address, bool>::iterator PPItr;
-	
+	typedef QMap<Uint32, Peer::Ptr> PeerMap;
+
+	static ConnectionLimit climit;
 
-	
 	class PeerManager::Private
 	{
 	public:
 		Private(PeerManager* p, Torrent & tor);
 		~Private();
-		
+
 		void updateAvailableChunks();
 		bool killBadPeer();
-		void createPeer(mse::EncryptedPacketSocket::Ptr sock,const PeerID & peer_id,Uint32 support,bool local);
+		void createPeer(mse::EncryptedPacketSocket::Ptr sock, const PeerID & peer_id, Uint32 support, bool local, ConnectionLimit::Token::Ptr token);
 		bool connectedTo(const net::Address & addr) const;
 		void update();
-		void have(Peer* peer,Uint32 index);
+		void have(Peer* peer, Uint32 index);
 		void connectToPeers();
-		
+
 	public:
 		PeerManager* p;
-		PtrMap<Uint32,Peer> peer_map;
-		QList<Peer*> peer_list;
-		QList<Peer*> killed;
+		QMap<Uint32, Peer::Ptr> peer_map;
 		Torrent & tor;
 		bool started;
 		BitSet available_chunks, wanted_chunks;
-		ChunkCounter* cnt;
+		ChunkCounter cnt;
 		bool pex_on;
 		bool wanted_changed;
 		PieceHandler* piece_handler;
 		bool paused;
 		QSet<PeerConnector::Ptr> connectors;
-		SuperSeeder* superseeder;
+		QScopedPointer<SuperSeeder> superseeder;
 		std::map<net::Address, bool> potential_peers;
 		bool partial_seed;
+		Uint32 num_cleared;
 	};
 
 	PeerManager::PeerManager(Torrent & tor)
-		: d(new Private(this,tor))
+		: d(new Private(this, tor))
 	{
 	}
 
@@ -105,13 +102,19 @@ namespace bt
 	{
 		delete d;
 	}
+	
+	ConnectionLimit& PeerManager::connectionLimits()
+	{
+		return climit;
+	}
+
 
 	void PeerManager::pause()
 	{
-		if (d->paused)
+		if(d->paused)
 			return;
-		
-		foreach (Peer* p,d->peer_list)
+
+		foreach(Peer::Ptr p, d->peer_map)
 		{
 			p->pause();
 		}
@@ -120,13 +123,13 @@ namespace bt
 
 	void PeerManager::unpause()
 	{
-		if (!d->paused)
+		if(!d->paused)
 			return;
-		
-		foreach (Peer* p,d->peer_list)
+
+		foreach(Peer::Ptr p, d->peer_map)
 		{
 			p->unpause();
-			if (p->hasWantedChunks(d->wanted_chunks)) // send interested when it has wanted chunks
+			if(p->hasWantedChunks(d->wanted_chunks))  // send interested when it has wanted chunks
 				p->sendInterested();
 		}
 		d->paused = false;
@@ -136,12 +139,9 @@ namespace bt
 	{
 		d->update();
 	}
-	
-	void PeerManager::setMaxConnections(Uint32 max)
-	{
-		max_connections = max;
-	}
-	
+
+#if 0
+
 	void PeerManager::setMaxTotalConnections(Uint32 max)
 	{
 #ifndef Q_WS_WIN
@@ -150,19 +150,20 @@ namespace bt
 		Uint32 sys_max = 9999; // there isn't a real limit on windows
 #endif
 		max_total_connections = max;
-		if (max == 0 || max_total_connections > sys_max)
+		if(max == 0 || max_total_connections > sys_max)
 			max_total_connections = sys_max;
 	}
-	
+#endif
+
 	void PeerManager::setWantedChunks(const BitSet & bs)
 	{
 		d->wanted_chunks = bs;
 		d->wanted_changed = true;
 	}
-	
+
 	void PeerManager::addPotentialPeer(const net::Address & addr, bool local)
 	{
-		if (d->potential_peers.size() < 500)
+		if(d->potential_peers.size() < 500)
 		{
 			d->potential_peers[addr] = local;
 		}
@@ -170,393 +171,359 @@ namespace bt
 
 	void PeerManager::killSeeders()
 	{
-		QList<Peer*>::iterator i = d->peer_list.begin();
-		while (i != d->peer_list.end())
+		foreach(Peer::Ptr peer, d->peer_map)
 		{
-			Peer* p = *i;
- 			if ( p->isSeeder() )
- 				p->kill();
-			i++;
-		}
-	}
-	
-	void PeerManager::killUninterested()
-	{
-		QList<Peer*>::iterator i = d->peer_list.begin();
-		while (i != d->peer_list.end())
-		{
-			Peer* p = *i;
-			if ( !p->isInterested() && (p->getConnectTime().secsTo(QTime::currentTime()) > 30) )
-				p->kill();
-			i++;
+			if(peer->isSeeder())
+				peer->kill();
 		}
 	}
 
-	void PeerManager::have(Peer* p,Uint32 index)
+	void PeerManager::killUninterested()
 	{
-		d->have(p,index);
+		QTime now = QTime::currentTime();
+		foreach(Peer::Ptr peer, d->peer_map)
+		{
+			if(!peer->isInterested() && (peer->getConnectTime().secsTo(now) > 30))
+				peer->kill();
+		}
+	}
+
+	void PeerManager::have(Peer* p, Uint32 index)
+	{
+		d->have(p, index);
 	}
 
 	void PeerManager::bitSetReceived(Peer* p, const BitSet & bs)
 	{
 		bool interested = false;
-		for (Uint32 i = 0;i < bs.getNumBits();i++)
+		for(Uint32 i = 0; i < bs.getNumBits(); i++)
 		{
-			if (bs.get(i))
+			if(bs.get(i))
 			{
-				if (d->wanted_chunks.get(i))
+				if(d->wanted_chunks.get(i))
 					interested = true;
-				d->available_chunks.set(i,true);
-				d->cnt->inc(i);
+				d->available_chunks.set(i, true);
+				d->cnt.inc(i);
 			}
 		}
-		
-		if (interested && !d->paused)
-			p->sendInterested();
-		
-		if (d->superseeder)
-			d->superseeder->bitset(p,bs);
-	}
-	
 
-	void PeerManager::newConnection(mse::EncryptedPacketSocket::Ptr sock,const PeerID & peer_id,Uint32 support)
+		if(interested && !d->paused)
+			p->sendInterested();
+
+		if(d->superseeder)
+			d->superseeder->bitset(p, bs);
+	}
+
+
+	void PeerManager::newConnection(mse::EncryptedPacketSocket::Ptr sock, const PeerID & peer_id, Uint32 support)
 	{
-		if (!d->started)
+		if(!d->started)
 			return;
 		
-		Uint32 total = d->peer_list.count() + d->connectors.size();
-		bool local_not_ok = (max_connections > 0 && total >= max_connections);
-		bool global_not_ok = (max_total_connections > 0 && total_connections >= max_total_connections);
-		
-		if (local_not_ok || global_not_ok)
+		ConnectionLimit::Token::Ptr token = climit.acquire(d->tor.getInfoHash());
+		if(!token && d->killBadPeer())
 		{
-			// get rid of bad peer and replace it by another one
-			if (!d->killBadPeer())
-			{
-				// we failed to find a bad peer, so just delete this one
-				return;
-			}
+			token = climit.acquire(d->tor.getInfoHash());
+			if(token)
+				d->createPeer(sock, peer_id, support, false, token);
+		}
+	}
+
+	void PeerManager::peerAuthenticated(bt::Authenticate* auth, bt::PeerConnector::WPtr pcon, bool ok, bt::ConnectionLimit::Token::Ptr token)
+	{
+		if(d->started)
+		{
+			if(ok && !connectedTo(auth->getPeerID()))
+				d->createPeer(auth->getSocket(), auth->getPeerID(), auth->supportedExtensions(), auth->isLocal(), token);
 		}
 
-		d->createPeer(sock,peer_id,support,false);
-	}
-	
-	void PeerManager::peerAuthenticated(Authenticate* auth,PeerConnector::WPtr pcon,bool ok)
-	{
-		if (d->started)
-		{
-			if (total_connections > 0)
-				total_connections--;
-			
-			if (ok && !connectedTo(auth->getPeerID()))
-				d->createPeer(auth->getSocket(),auth->getPeerID(),auth->supportedExtensions(),auth->isLocal());
-		}
-		
 		PeerConnector::Ptr ptr = pcon.toStrongRef();
 		d->connectors.remove(ptr);
 	}
-	
 
-		
+
+
 	bool PeerManager::connectedTo(const PeerID & peer_id)
 	{
-		if (!d->started)
+		if(!d->started)
 			return false;
-		
-		foreach (Peer* p,d->peer_list)
+
+		foreach(Peer::Ptr p, d->peer_map)
 		{
-			if (p->getPeerID() == peer_id)
-			{
+			if(p->getPeerID() == peer_id)
 				return true;
-			}
 		}
 		return false;
 	}
-	
-	void PeerManager::connectToPeers()
-	{
-		d->connectToPeers();
-	}
-	
-	Uint32 PeerManager::clearDeadPeers()
-	{
-		Uint32 num = d->killed.count();
-		qDeleteAll(d->killed);
-		d->killed.clear();
-		return num;
-	}
-	
+
 	void PeerManager::closeAllConnections()
 	{
-		qDeleteAll(d->killed);
-		d->killed.clear();
-	
-		if ((Uint32)d->peer_list.count() <= total_connections)
-			total_connections -= d->peer_list.count();
-		else
-			total_connections = 0;
-
 		d->peer_map.clear();
-		qDeleteAll(d->peer_list);
-		d->peer_list.clear();
 	}
-	
-	
+
+
 	void PeerManager::savePeerList(const QString & file)
 	{
 		// Lets save the entries line based
 		QFile fptr(file);
-		if (!fptr.open(QIODevice::WriteOnly))
+		if(!fptr.open(QIODevice::WriteOnly))
 			return;
-		
-		
+
 		try
 		{
-			Out(SYS_GEN|LOG_DEBUG) << "Saving list of peers to " << file << endl;
-			
+			Out(SYS_GEN | LOG_DEBUG) << "Saving list of peers to " << file << endl;
+
 			QTextStream out(&fptr);
 			// first the active peers
-			foreach(Peer* p,d->peer_list)
+			foreach(Peer::Ptr p, d->peer_map)
 			{
 				const net::Address & addr = p->getAddress();
 				out << addr.toString() << " " << (unsigned short)addr.port() << ::endl;
 			}
-			
+
 			// now the potential_peers
 			PPItr i = d->potential_peers.begin();
-			while (i != d->potential_peers.end())
+			while(i != d->potential_peers.end())
 			{
 				out << i->first.toString() << " " <<  i->first.port() << ::endl;
 				i++;
 			}
 		}
-		catch (bt::Error & err)
+		catch(bt::Error & err)
 		{
-			Out(SYS_GEN|LOG_DEBUG) << "Error happened during saving of peer list : " << err.toString() << endl;
+			Out(SYS_GEN | LOG_DEBUG) << "Error happened during saving of peer list : " << err.toString() << endl;
 		}
 	}
-	
+
 	void PeerManager::loadPeerList(const QString & file)
 	{
 		QFile fptr(file);
-		if (!fptr.open(QIODevice::ReadOnly))
+		if(!fptr.open(QIODevice::ReadOnly))
 			return;
-		
+
 		try
 		{
-			
-			Out(SYS_GEN|LOG_DEBUG) << "Loading list of peers from " << file  << endl;
-			
-			while (!fptr.atEnd())
+
+			Out(SYS_GEN | LOG_DEBUG) << "Loading list of peers from " << file  << endl;
+
+			while(!fptr.atEnd())
 			{
 				QStringList sl = QString(fptr.readLine()).split(" ");
-				if (sl.count() != 2)
+				if(sl.count() != 2)
 					continue;
-				
+
 				bool ok = false;
 				net::Address addr(sl[0], sl[1].toInt(&ok));
-				if (ok)
+				if(ok)
 					addPotentialPeer(addr, false);
 			}
 		}
-		catch (bt::Error & err)
+		catch(bt::Error & err)
 		{
-			Out(SYS_GEN|LOG_DEBUG) << "Error happened during saving of peer list : " << err.toString() << endl;
+			Out(SYS_GEN | LOG_DEBUG) << "Error happened during saving of peer list : " << err.toString() << endl;
 		}
 	}
-	
+
 	void PeerManager::start(bool superseed)
 	{
 		d->started = true;
-		if (superseed && !d->superseeder)
-			d->superseeder = new SuperSeeder(d->cnt->getNumChunks(),this);
-		
+		if(superseed && !d->superseeder)
+			d->superseeder.reset(new SuperSeeder(d->cnt.getNumChunks()));
+
 		unpause();
 		ServerInterface::addPeerManager(this);
 	}
-		
-	
+
+
 	void PeerManager::stop()
 	{
-		d->cnt->reset();
+		d->cnt.reset();
 		d->available_chunks.clear();
 		d->started = false;
 		ServerInterface::removePeerManager(this);
 		d->connectors.clear();
-		
-		delete d->superseeder;
-		d->superseeder = 0;
-		
+		d->superseeder.reset();
 		closeAllConnections();
 	}
 
-	Peer* PeerManager::findPeer(Uint32 peer_id)
+	Peer::Ptr PeerManager::findPeer(Uint32 peer_id)
 	{
-		return d->peer_map.find(peer_id);
+		PeerMap::iterator i = d->peer_map.find(peer_id);
+		if(i == d->peer_map.end())
+			return Peer::Ptr();
+		else
+			return *i;
 	}
-	
-	Peer* PeerManager::findPeer(PieceDownloader* pd)
+
+	Peer::Ptr PeerManager::findPeer(PieceDownloader* pd)
 	{
-		foreach (Peer* p,d->peer_list)
+		foreach(Peer::Ptr p, d->peer_map)
 		{
-			if ((PieceDownloader*)p->getPeerDownloader() == pd)
+			if((PieceDownloader*)p->getPeerDownloader() == pd)
 				return p;
 		}
-		return 0;
+		return Peer::Ptr();
 	}
-	
+
 	void PeerManager::rerunChoker()
 	{
-		// append a 0 ptr to killed
-		// so that the next update in TorrentControl
-		// will be forced to do the choking
-		d->killed.append(0);
+		d->num_cleared++;
 	}
-	
-	
-	
+
+	bool PeerManager::chokerNeedsToRun() const
+	{
+		return d->num_cleared > 0;
+	}
+
 	void PeerManager::peerSourceReady(PeerSource* ps)
 	{
 		net::Address addr;
 		bool local = false;
-		while (ps->takePeer(addr, local))
+		while(ps->takePeer(addr, local))
 			addPotentialPeer(addr, local);
 	}
-	
-	
-	
+
+
+
 	void PeerManager::pex(const QByteArray & arr)
 	{
-		if (!d->pex_on)
+		if(!d->pex_on)
 			return;
-		
-		Out(SYS_CON|LOG_NOTICE) << "PEX: found " << (arr.size() / 6) << " peers"  << endl;
-		for (int i = 0;i+6 <= arr.size();i+=6)
+
+		Out(SYS_CON | LOG_NOTICE) << "PEX: found " << (arr.size() / 6) << " peers"  << endl;
+		for(int i = 0; i + 6 <= arr.size(); i += 6)
 		{
 			const Uint8* tmp = (const Uint8*)arr.data() + i;
-			addPotentialPeer(net::Address(ReadUint32(tmp,0), ReadUint16(tmp,4)), false);
+			addPotentialPeer(net::Address(ReadUint32(tmp, 0), ReadUint16(tmp, 4)), false);
 		}
 	}
-	
-	
+
+
 	void PeerManager::setPexEnabled(bool on)
 	{
-		if (on && d->tor.isPrivate())
+		if(on && d->tor.isPrivate())
 			return;
-		
-		if (d->pex_on == on)
+
+		if(d->pex_on == on)
 			return;
-		
-		foreach (Peer* p,d->peer_list)
+
+		foreach(Peer::Ptr p, d->peer_map)
 		{
-			if (!p->isKilled())
+			if(!p->isKilled())
 			{
 				p->setPexEnabled(on);
 				bt::Uint16 port = ServerInterface::getPort();
-				p->sendExtProtHandshake(port,d->tor.getMetaData().size(),d->partial_seed);
+				p->sendExtProtHandshake(port, d->tor.getMetaData().size(), d->partial_seed);
 			}
 		}
 		d->pex_on = on;
 	}
-	
-	void PeerManager::setGroupIDs(Uint32 up,Uint32 down)
+
+	void PeerManager::setGroupIDs(Uint32 up, Uint32 down)
 	{
-		for (PtrMap<Uint32,Peer>::iterator i = d->peer_map.begin();i != d->peer_map.end();i++)
-		{
-			Peer* p = i->second;
-			p->setGroupIDs(up,down);
-		}
+		for(PeerMap::iterator i = d->peer_map.begin(); i != d->peer_map.end(); i++)
+			i.value()->setGroupIDs(up, down);
 	}
-		
+
 	void PeerManager::portPacketReceived(const QString& ip, Uint16 port)
 	{
-		if (Globals::instance().getDHT().isRunning() && !d->tor.isPrivate())
-			Globals::instance().getDHT().portReceived(ip,port);
+		if(Globals::instance().getDHT().isRunning() && !d->tor.isPrivate())
+			Globals::instance().getDHT().portReceived(ip, port);
 	}
-	
+
 	void PeerManager::pieceReceived(const Piece & p)
 	{
-		if (d->piece_handler)
+		if(d->piece_handler)
 			d->piece_handler->pieceReceived(p);
 	}
-		
+
 	void PeerManager::setPieceHandler(PieceHandler* ph)
 	{
 		d->piece_handler = ph;
 	}
-	
+
 	void PeerManager::killStalePeers()
 	{
-		foreach (Peer* p,d->peer_list)
+		foreach(Peer::Ptr p, d->peer_map)
 		{
-			if (p->getDownloadRate() == 0 && p->getUploadRate() == 0)
+			if(p->getDownloadRate() == 0 && p->getUploadRate() == 0)
 				p->kill();
 		}
 	}
-	
-	void PeerManager::setSuperSeeding(bool on,const BitSet & chunks)
+
+	void PeerManager::setSuperSeeding(bool on, const BitSet & chunks)
 	{
 		Q_UNUSED(chunks);
-		if ((d->superseeder && on) || (!d->superseeder && !on))
+		if((d->superseeder && on) || (!d->superseeder && !on))
 			return;
-		
-		if (on)
-		{
-			d->superseeder = new SuperSeeder(d->cnt->getNumChunks(),this);
-		}
-		else
-		{
-			delete d->superseeder;
-			d->superseeder = 0;
-		}
-		
-		// When entering or exiting superseeding mode kill all peers 
+
+		d->superseeder.reset(on ? new SuperSeeder(d->cnt.getNumChunks()) : 0);
+
+		// When entering or exiting superseeding mode kill all peers
 		// but first add the current list to the potential_peers list, so we can reconnect later.
-		foreach(Peer* p,d->peer_list)
+		foreach(Peer::Ptr p, d->peer_map)
 		{
 			const net::Address & addr = p->getAddress();
 			addPotentialPeer(addr, false);
 			p->kill();
 		}
 	}
-	
-	void PeerManager::allowChunk(PeerInterface* peer, Uint32 chunk)
-	{
-		Out(SYS_CON|LOG_DEBUG) << "Peer " << peer->getPeerID().toString() << " is allowed to download " << chunk << endl;
-		Peer* p = dynamic_cast<Peer*>(peer);
-		if (p)
-			p->sendHave(chunk);
-	}
-	
+
 	void PeerManager::sendHave(Uint32 index)
 	{
-		if (d->superseeder)
+		if(d->superseeder)
 			return;
-		
-		foreach (Peer* peer, d->peer_list)
+
+		foreach(Peer::Ptr peer, d->peer_map)
 		{
 			peer->sendHave(index);
 		}
 	}
-	
-	Uint32 PeerManager::getNumConnectedPeers() const 
+
+	Uint32 PeerManager::getNumConnectedPeers() const
 	{
-		return d->peer_list.count();
+		return d->peer_map.count();
 	}
-	
-	Uint32 PeerManager::getNumPending() const 
+
+	Uint32 PeerManager::getNumConnectedLeechers() const
+	{
+		Uint32 cnt = 0;
+		foreach(Peer::Ptr peer, d->peer_map)
+		{
+			if(!peer->isSeeder())
+				cnt++;
+		}
+
+		return cnt;
+	}
+
+	Uint32 PeerManager::getNumConnectedSeeders() const
+	{
+		Uint32 cnt = 0;
+		foreach(Peer::Ptr peer, d->peer_map)
+		{
+			if(peer->isSeeder())
+				cnt++;
+		}
+
+		return cnt;
+	}
+
+
+
+	Uint32 PeerManager::getNumPending() const
 	{
 		return d->connectors.size();
 	}
-	
+
 	const bt::BitSet& PeerManager::getAvailableChunksBitSet() const
 	{
 		return d->available_chunks;
 	}
-	
+
 	ChunkCounter& PeerManager::getChunkCounter()
 	{
-		return *d->cnt;
+		return d->cnt;
 	}
 
 	bool PeerManager::isPexEnabled() const
@@ -568,7 +535,7 @@ namespace bt
 	{
 		return d->tor;
 	}
-	
+
 	bool PeerManager::isStarted() const
 	{
 		return d->started;
@@ -576,36 +543,42 @@ namespace bt
 
 	void PeerManager::visit(PeerManager::PeerVisitor& visitor)
 	{
-		foreach (const Peer* p,d->peer_list)
+		foreach(const Peer::Ptr p, d->peer_map)
+		{
 			visitor.visit(p);
+		}
 	}
-	
+
 	Uint32 PeerManager::uploadRate() const
 	{
 		Uint32 rate = 0;
-		foreach (const Peer* p,d->peer_list)
+		foreach(const Peer::Ptr p, d->peer_map)
+		{
 			rate += p->getUploadRate();
+		}
 		return rate;
 	}
 
-	QList<Peer*> PeerManager::getPeers() const
+	QList<Peer::Ptr> PeerManager::getPeers() const
 	{
-		return d->peer_list;
+		return d->peer_map.values();
 	}
-	
+
 	void PeerManager::setPartialSeed(bool partial_seed)
 	{
-		if (d->partial_seed != partial_seed)
+		if(d->partial_seed != partial_seed)
 		{
 			d->partial_seed = partial_seed;
-			
+
 			// If partial seeding status changes, update all peers
 			bt::Uint16 port = ServerInterface::getPort();
-			foreach (Peer* peer,d->peer_list)
-				peer->sendExtProtHandshake(port,d->tor.getMetaData().size(),d->partial_seed);
+			foreach(Peer::Ptr peer, d->peer_map)
+			{
+				peer->sendExtProtHandshake(port, d->tor.getMetaData().size(), d->partial_seed);
+			}
 		}
 	}
-	
+
 	bool PeerManager::isPartialSeed() const
 	{
 		return d->partial_seed;
@@ -613,208 +586,165 @@ namespace bt
 
 
 	//////////////////////////////////////////////////
-	
+
 	PeerManager::Private::Private(PeerManager* p, Torrent& tor)
 		: p(p),
-		tor(tor),
-		available_chunks(tor.getNumChunks()),
-		wanted_chunks(tor.getNumChunks()),
-		partial_seed(false)
+		  tor(tor),
+		  available_chunks(tor.getNumChunks()),
+		  wanted_chunks(tor.getNumChunks()),
+		  cnt(tor.getNumChunks()),
+		  partial_seed(false),
+		  num_cleared(0)
 	{
 		started = false;
 		wanted_chunks.setAll(true);
 		wanted_changed = false;
-		cnt = new ChunkCounter(tor.getNumChunks());
 		pex_on = !tor.isPrivate();
 		piece_handler = 0;
 		paused = false;
-		superseeder = 0;
 	}
 
 	PeerManager::Private::~Private()
 	{
-		delete cnt;
 		ServerInterface::removePeerManager(p);
-		
-		if ((Uint32)peer_list.count() <= total_connections)
-			total_connections -= peer_list.count();
-		else
-			total_connections = 0;
-		
-		qDeleteAll(peer_list.begin(),peer_list.end());
-		peer_list.clear();
-		
-		delete superseeder;
-		
 		started = false;
 		connectors.clear();
 	}
 
 	void PeerManager::Private::update()
 	{
-		if (!started)
+		if(!started)
 			return;
-		
+
+		num_cleared = 0;
+
 		// update the speed of each peer,
 		// and get rid of some killed peers
-		QList<Peer*>::iterator i = peer_list.begin();
-		while (i != peer_list.end())
+		for(PeerMap::iterator i = peer_map.begin(); i != peer_map.end();)
 		{
-			Peer* peer = *i;
-			if (!peer->isKilled() && peer->isStalled())
+			Peer::Ptr peer = i.value();
+			if(!peer->isKilled() && peer->isStalled())
 			{
 				p->addPotentialPeer(peer->getAddress(), peer->getStats().local);
 				peer->kill();
 			}
-			
-			if (peer->isKilled())
+
+			if(peer->isKilled())
 			{
-				cnt->decBitSet(peer->getBitSet());
+				cnt.decBitSet(peer->getBitSet());
 				updateAvailableChunks();
-				i = peer_list.erase(i);
-				killed.append(peer);
-				peer_map.erase(peer->getID());
-				if (total_connections > 0)
-					total_connections--;
-				p->peerKilled(peer);
-				if (superseeder)
-					superseeder->peerRemoved(peer);
+				i = peer_map.erase(i);
+				p->peerKilled(peer.data());
+				if(superseeder)
+					superseeder->peerRemoved(peer.data());
+				num_cleared++;
 			}
 			else
 			{
 				peer->update();
+				if(wanted_changed)
+				{
+					if(peer->hasWantedChunks(wanted_chunks))
+						peer->sendInterested();
+					else
+						peer->sendNotInterested();
+				}
 				i++;
 			}
 		}
-		if (wanted_changed)
-		{
-			foreach (Peer* peer,peer_list)
-			{
-				if (peer->hasWantedChunks(wanted_chunks))
-					peer->sendInterested();
-				else
-					peer->sendNotInterested();
-				i++;
-			}
-			wanted_changed = false;
-		}
+
+		wanted_changed = false;
+		connectToPeers();
 	}
 
 	void PeerManager::Private::have(Peer* peer, Uint32 index)
 	{
-		if (wanted_chunks.get(index) && !paused)
+		if(wanted_chunks.get(index) && !paused)
 			peer->sendInterested();
-		available_chunks.set(index,true);
-		cnt->inc(index);
-		if (superseeder)
-			superseeder->have(peer,index);
+		available_chunks.set(index, true);
+		cnt.inc(index);
+		if(superseeder)
+			superseeder->have(peer, index);
 	}
-	
+
 	void PeerManager::Private::updateAvailableChunks()
 	{
-		for (Uint32 i = 0;i < available_chunks.getNumBits();i++)
+		for(Uint32 i = 0; i < available_chunks.getNumBits(); i++)
 		{
-			available_chunks.set(i,cnt->get(i) > 0);
+			available_chunks.set(i, cnt.get(i) > 0);
 		}
 	}
-	
+
 	bool PeerManager::Private::killBadPeer()
 	{
-		for (PtrMap<Uint32,Peer>::iterator i = peer_map.begin();i != peer_map.end();i++)
+		for(PeerMap::iterator i = peer_map.begin(); i != peer_map.end(); i++)
 		{
-			Peer* peer = i->second;
-			if (peer->getStats().aca_score <= -5.0 && peer->getStats().aca_score > -50.0)
+			Peer::Ptr peer = i.value();
+			if(peer->getStats().aca_score <= -5.0 && peer->getStats().aca_score > -50.0)
 			{
-				Out(SYS_GEN|LOG_DEBUG) << "Killing bad peer, to make room for other peers" << endl;
+				Out(SYS_GEN | LOG_DEBUG) << "Killing bad peer, to make room for other peers" << endl;
 				peer->kill();
 				return true;
 			}
 		}
 		return false;
 	}
-	
-	void PeerManager::Private::createPeer(mse::EncryptedPacketSocket::Ptr sock,const PeerID & peer_id,Uint32 support,bool local)
+
+	void PeerManager::Private::createPeer(mse::EncryptedPacketSocket::Ptr sock, const bt::PeerID& peer_id, Uint32 support, bool local, bt::ConnectionLimit::Token::Ptr token)
 	{
-		Peer* peer = new Peer(sock,peer_id,tor.getNumChunks(),tor.getChunkSize(),support,local,p);
-		peer_list.append(peer);
-		peer_map.insert(peer->getID(),peer);
-		total_connections++;
-		p->newPeer(peer);
+		Peer::Ptr peer(new Peer(sock, peer_id, tor.getNumChunks(), tor.getChunkSize(), support, local, token, p));
+		peer_map.insert(peer->getID(), peer);
+		p->newPeer(peer.data());
 		peer->setPexEnabled(pex_on);
 		// send extension protocol handshake
 		bt::Uint16 port = ServerInterface::getPort();
-		peer->sendExtProtHandshake(port,tor.getMetaData().size(),partial_seed);
-		
-		if (superseeder)
-			superseeder->peerAdded(peer);
+		peer->sendExtProtHandshake(port, tor.getMetaData().size(), partial_seed);
+
+		if(superseeder)
+			superseeder->peerAdded(peer.data());
 	}
-	
-	
+
+
 	bool PeerManager::Private::connectedTo(const net::Address & addr) const
 	{
-		PtrMap<Uint32,Peer>::const_iterator i = peer_map.begin();
-		while (i != peer_map.end())
+		PeerMap::const_iterator i = peer_map.begin();
+		while(i != peer_map.end())
 		{
-			const Peer* peer = i->second;
-			if (peer->getAddress() == addr)
+			if(i.value()->getAddress() == addr)
 				return true;
 			i++;
 		}
 		return false;
 	}
-	
+
 	void PeerManager::Private::connectToPeers()
 	{
-		if (paused)
+		if(paused || potential_peers.size() == 0 || (Uint32)connectors.size() > MAX_SIMULTANIOUS_AUTHS)
 			return;
-		
-		if (potential_peers.size() == 0)
-			return;
-		
-		if (peer_list.count() + connectors.size() >= (int)max_connections && max_connections > 0)
-			return;
-		
-		if (total_connections >= max_total_connections && max_total_connections > 0)
-			return;
-		
-		if ((Uint32)connectors.size() > MAX_SIMULTANIOUS_AUTHS)
-			return;
-		
-		Uint32 num = 0;
-		if (max_connections > 0)
+
+		while(!potential_peers.empty())
 		{
-			Uint32 available = max_connections - (peer_list.count() + connectors.size());
-			num = qMin<Uint32>(available, potential_peers.size());
-		}
-		else
-		{
-			num = potential_peers.size();
-		}
-		
-		if (num + total_connections >= max_total_connections && max_total_connections > 0)
-			num = max_total_connections - total_connections;
-		
-		for (Uint32 i = 0;i < num && !potential_peers.empty();i++)
-		{
-			if ((Uint32)connectors.size() > MAX_SIMULTANIOUS_AUTHS)
+			if((Uint32)connectors.size() > MAX_SIMULTANIOUS_AUTHS)
 				return;
 			
 			PPItr itr = potential_peers.begin();
-			
+
 			AccessManager & aman = AccessManager::instance();
-			
-			if (aman.allowed(itr->first) && !connectedTo(itr->first))
+
+			if(aman.allowed(itr->first) && !connectedTo(itr->first))
 			{
-				PeerConnector::Ptr pcon(new PeerConnector(itr->first,itr->second,p));
+				ConnectionLimit::Token::Ptr token = climit.acquire(tor.getInfoHash());
+				if(!token)
+					break;
+
+				PeerConnector::Ptr pcon(new PeerConnector(itr->first, itr->second, p, token));
 				pcon->setWeakPointer(PeerConnector::WPtr(pcon));
 				connectors.insert(pcon);
-				total_connections++;
 				pcon->start();
 			}
+
 			potential_peers.erase(itr);
 		}
 	}
 
 }
-
-#include "peermanager.moc"
