@@ -19,10 +19,9 @@
  ***************************************************************************/
 #include "httptracker.h"
 #include <config-ktorrent.h>
- 
+
 #include <kurl.h>
 #include <klocale.h>
-#include <k3socketaddress.h>
 #include <qhostaddress.h>
 #include <util/log.h>
 #include <util/functions.h>
@@ -51,28 +50,28 @@ namespace bt
 	Uint16 HTTPTracker::proxy_port = 8080;
 	bool HTTPTracker::use_qhttp = false;
 
-	HTTPTracker::HTTPTracker(const KUrl & url,TrackerDataSource* tds,const PeerID & id,int tier)
-		: Tracker(url,tds,id,tier)
+	HTTPTracker::HTTPTracker(const KUrl & url, TrackerDataSource* tds, const PeerID & id, int tier)
+			: Tracker(url, tds, id, tier),
+			supports_partial_seed_extension(false)
 	{
 		active_job = 0;
-		
+
 		interval = 5 * 60; // default interval 5 minutes
 		failures = 0;
-		connect(&timer,SIGNAL(timeout()),this,SLOT(onTimeout()));
-		
+		connect(&timer, SIGNAL(timeout()), this, SLOT(onTimeout()));
 	}
 
 	HTTPTracker::~HTTPTracker()
 	{
 	}
-	
+
 	void HTTPTracker::start()
 	{
 		event = "started";
 		resetTrackerStats();
 		doRequest();
 	}
-	
+
 	void HTTPTracker::stop(WaitJob* wjob)
 	{
 		if (!started)
@@ -95,14 +94,14 @@ namespace bt
 			started = false;
 		}
 	}
-	
+
 	void HTTPTracker::completed()
 	{
 		event = "completed";
 		doRequest();
 		event = QString();
 	}
-	
+
 	void HTTPTracker::manualUpdate()
 	{
 		if (!started)
@@ -110,24 +109,24 @@ namespace bt
 		else
 			doRequest();
 	}
-	
+
 	void HTTPTracker::scrape()
 	{
 		if (!url.isValid())
 		{
-			Out(SYS_TRK|LOG_NOTICE) << "Invalid tracker url, canceling scrape" << endl;
+			Out(SYS_TRK | LOG_NOTICE) << "Invalid tracker url, canceling scrape" << endl;
 			return;
 		}
-		
+
 		if (!url.fileName().startsWith("announce"))
 		{
-			Out(SYS_TRK|LOG_NOTICE) << "Tracker " << url << " does not support scraping" << endl;
+			Out(SYS_TRK | LOG_NOTICE) << "Tracker " << url << " does not support scraping" << endl;
 			return;
 		}
-		
+
 		KUrl scrape_url = url;
-		scrape_url.setFileName(url.fileName().replace("announce","scrape"));
-		
+		scrape_url.setFileName(url.fileName().replace("announce", "scrape"));
+
 		QString epq = scrape_url.encodedPathAndQuery();
 		const SHA1Hash & info_hash = tds->infoHash();
 		if (scrape_url.queryItems().count() > 0)
@@ -135,44 +134,44 @@ namespace bt
 		else
 			epq += "?info_hash=" + info_hash.toURLString();
 		scrape_url.setEncodedPathAndQuery(epq);
-	
-		Out(SYS_TRK|LOG_NOTICE) << "Doing scrape request to url : " << scrape_url.prettyUrl() << endl;
+
+		Out(SYS_TRK | LOG_NOTICE) << "Doing scrape request to url : " << scrape_url.prettyUrl() << endl;
 		KIO::MetaData md;
 		setupMetaData(md);
-		
+
 		KIO::StoredTransferJob* j = KIO::storedGet(scrape_url, KIO::NoReload, KIO::HideProgressInfo);
 		// set the meta data
 		j->setMetaData(md);
 		KIO::Scheduler::scheduleJob(j);
-		
-		connect(j,SIGNAL(result(KJob* )),this,SLOT(onScrapeResult( KJob* )));
+
+		connect(j, SIGNAL(result(KJob*)), this, SLOT(onScrapeResult(KJob*)));
 	}
-	
+
 	void HTTPTracker::onScrapeResult(KJob* j)
 	{
 		if (j->error())
 		{
-			Out(SYS_TRK|LOG_IMPORTANT) << "Scrape failed : " << j->errorString() << endl;
+			Out(SYS_TRK | LOG_IMPORTANT) << "Scrape failed : " << j->errorString() << endl;
 			return;
 		}
-		
+
 		KIO::StoredTransferJob* st = (KIO::StoredTransferJob*)j;
-		BDecoder dec(st->data(),false,0);
-		BNode* n = 0;
-		
+		BDecoder dec(st->data(), false, 0);
+		QScopedPointer<BNode> n;
+
 		try
 		{
-			n = dec.decode();
+			n.reset(dec.decode());
 		}
 		catch (bt::Error & err)
 		{
-			Out(SYS_TRK|LOG_IMPORTANT) << "Invalid scrape data " << err.toString() << endl;
+			Out(SYS_TRK | LOG_IMPORTANT) << "Invalid scrape data " << err.toString() << endl;
 			return;
 		}
-			
+
 		if (n && n->getType() == BNode::DICT)
 		{
-			BDictNode* d = (BDictNode*)n;
+			BDictNode* d = (BDictNode*)n.data();
 			d = d->getDict(QString("files"));
 			if (d)
 			{
@@ -184,11 +183,12 @@ namespace bt
 						seeders = d->getInt("complete");
 						leechers = d->getInt("incomplete");
 						total_downloaded = d->getInt("downloaded");
-						Out(SYS_TRK|LOG_DEBUG) << "Scrape : leechers = " << leechers 
-							<< ", seeders = " << seeders << ", downloaded = " << total_downloaded << endl;
+						supports_partial_seed_extension = d->getValue("downloaders") != 0;
+						Out(SYS_TRK | LOG_DEBUG) << "Scrape : leechers = " << leechers
+						<< ", seeders = " << seeders << ", downloaded = " << total_downloaded << endl;
 					}
 					catch (...)
-					{}
+						{}
 					scrapeDone();
 					if (status == bt::TRACKER_ERROR)
 					{
@@ -198,72 +198,77 @@ namespace bt
 				}
 			}
 		}
-		
-		delete n;
 	}
-	
+
 	void HTTPTracker::doRequest(WaitJob* wjob)
 	{
 		KUrl u = url;
 		if (!url.isValid())
 		{
 			requestPending();
-			QTimer::singleShot(500,this,SLOT(emitInvalidURLFailure()));
+			QTimer::singleShot(500, this, SLOT(emitInvalidURLFailure()));
 			return;
 		}
 
 		Uint16 port = ServerInterface::getPort();
-		
-		u.addQueryItem("peer_id",peer_id.toString());
-		u.addQueryItem("port",QString::number(port));
-		u.addQueryItem("uploaded",QString::number(bytesUploaded()));
-		u.addQueryItem("downloaded",QString::number(bytesDownloaded()));
-		
-		if (event == "completed")
-			u.addQueryItem("left","0"); // need to send 0 when we are completed
-		else
-			u.addQueryItem("left",QString::number(tds->bytesLeft()));
-		
-		u.addQueryItem("compact","1");
-		if (event != "stopped")
-			u.addQueryItem("numwant","200");
-		else
-			u.addQueryItem("numwant","0");
-		
-		u.addQueryItem("key",QString::number(key));
-		QString cip = Tracker::getCustomIP();
-		if (!cip.isNull())
-			u.addQueryItem("ip",cip);
 
-		if (event != QString())
-			u.addQueryItem("event",event);
+		u.addQueryItem("peer_id", peer_id.toString());
+		u.addQueryItem("port", QString::number(port));
+		u.addQueryItem("uploaded", QString::number(bytesUploaded()));
+		u.addQueryItem("downloaded", QString::number(bytesDownloaded()));
+
+		if (event == "completed")
+			u.addQueryItem("left", "0"); // need to send 0 when we are completed
+		else
+			u.addQueryItem("left", QString::number(tds->bytesLeft()));
+
+		u.addQueryItem("compact", "1");
+		if (event != "stopped")
+			u.addQueryItem("numwant", "200");
+		else
+			u.addQueryItem("numwant", "0");
+
+		u.addQueryItem("key", QString::number(key));
+		QString cip = Tracker::getCustomIP();
+		if (cip.isNull())
+			cip = CurrentIPv6Address();
+		
+		if (!cip.isEmpty())
+			u.addQueryItem("ip", cip);
+		
+		if (event.isEmpty() && supports_partial_seed_extension && tds->isPartialSeed())
+			event = "paused";
+
+		if (!event.isEmpty())
+			u.addQueryItem("event", event);
+		
 		QString epq = u.encodedPathAndQuery();
 		const SHA1Hash & info_hash = tds->infoHash();
 		epq += "&info_hash=" + info_hash.toURLString();
 
 
 		u.setEncodedPathAndQuery(epq);
-		
+
 		if (active_job)
-  		{
- 			announce_queue.append(u);
- 			Out(SYS_TRK|LOG_NOTICE) << "Announce ongoing, queueing announce" << endl;
-  		}
- 		else
- 		{
- 			doAnnounce(u);
- 			// if there is a wait job, add this job to the waitjob 
- 			if (wjob)
- 				wjob->addExitOperation(new ExitJobOperation(active_job));
- 		}
+		{
+			announce_queue.append(u);
+			Out(SYS_TRK | LOG_NOTICE) << "Announce ongoing, queueing announce" << endl;
+		}
+		else
+		{
+			doAnnounce(u);
+			// if there is a wait job, add this job to the waitjob
+			if (wjob)
+				wjob->addExitOperation(new ExitJobOperation(active_job));
+		}
 	}
 
 	bool HTTPTracker::updateData(const QByteArray & data)
 	{
 //#define DEBUG_PRINT_RESPONSE
 #ifdef DEBUG_PRINT_RESPONSE
-		Out(SYS_TRK|LOG_DEBUG) << "Data : " << endl;
-		Out(SYS_TRK|LOG_DEBUG) << QString(data) << endl;
+		Out(SYS_TRK | LOG_DEBUG) << "Data : " << endl;
+		Out(SYS_TRK | LOG_DEBUG) << QString(data) << endl;
 #endif
 		// search for dictionary, there might be random garbage infront of the data
 		int i = 0;
@@ -273,15 +278,15 @@ namespace bt
 				break;
 			i++;
 		}
-		
+
 		if (i == data.size())
 		{
 			failures++;
 			failed(i18n("Invalid response from tracker"));
 			return false;
 		}
-		
-		BDecoder dec(data,false,i);
+
+		BDecoder dec(data, false, i);
 		BNode* n = 0;
 		try
 		{
@@ -293,14 +298,14 @@ namespace bt
 			failed(i18n("Invalid data from tracker"));
 			return false;
 		}
-			
+
 		if (!n || n->getType() != BNode::DICT)
 		{
 			failures++;
 			failed(i18n("Invalid response from tracker"));
 			return false;
 		}
-			
+
 		BDictNode* dict = (BDictNode*)n;
 		if (dict->getData("failure reason"))
 		{
@@ -311,15 +316,23 @@ namespace bt
 			failed(error);
 			return false;
 		}
-			
+
+		if (dict->getData("warning message"))
+		{
+			BValueNode* vn = dict->getValue("warning message");
+			warning = vn->data().toString();
+		}
+		else
+			warning.clear();
+
 		BValueNode* vn = dict->getValue("interval");
-			
+
 		// if no interval is specified, use 5 minutes
 		if (vn)
 			interval = vn->data().toInt();
 		else
 			interval = 5 * 60;
-			
+
 		vn = dict->getValue("incomplete");
 		if (vn)
 			leechers = vn->data().toInt();
@@ -327,7 +340,7 @@ namespace bt
 		vn = dict->getValue("complete");
 		if (vn)
 			seeders = vn->data().toInt();
-	
+
 		BListNode* ln = dict->getList("peers");
 		if (!ln)
 		{
@@ -336,20 +349,14 @@ namespace bt
 			if (vn && vn->data().getType() == Value::STRING)
 			{
 				QByteArray arr = vn->data().toByteArray();
-				for (int i = 0;i < arr.size();i+=6)
+				for (int i = 0;i < arr.size();i += 6)
 				{
 					Uint8 buf[6];
 					for (int j = 0;j < 6;j++)
 						buf[j] = arr[i + j];
 
-					Uint32 ip = ReadUint32(buf,0);
-					QString ip_str = QString("%1.%2.%3.%4")
-						.arg((ip & 0xFF000000) >> 24)
-						.arg((ip & 0x00FF0000) >> 16)
-						.arg((ip & 0x0000FF00) >> 8)
-						.arg(ip & 0x000000FF);
-
-					addPeer(ip_str,ReadUint16(buf,4));
+					Uint32 ip = ReadUint32(buf, 0);
+					addPeer(net::Address(ip, ReadUint16(buf, 4)), false);
 				}
 			}
 		}
@@ -361,52 +368,52 @@ namespace bt
 
 				if (!dict)
 					continue;
-				
+
 				BValueNode* ip_node = dict->getValue("ip");
 				BValueNode* port_node = dict->getValue("port");
 
 				if (!ip_node || !port_node)
 					continue;
-				
-				addPeer(ip_node->data().toString(),port_node->data().toInt());
+
+				net::Address addr(ip_node->data().toString(), port_node->data().toInt());
+				addPeer(addr, false);
 			}
 		}
-		
+
 		// Check for IPv6 compact peers
 		vn = dict->getValue("peers6");
 		if (vn && vn->data().getType() == Value::STRING)
 		{
 			QByteArray arr = vn->data().toByteArray();
-			for (int i = 0;i < arr.size();i+=18)
+			for (int i = 0;i < arr.size();i += 18)
 			{
-				Uint8 buf[18];
-				for (int j = 0;j < 18;j++)
-					buf[j] = arr[i + j];
+				Q_IPV6ADDR ip;
+				memcpy(ip.c, arr.data() + i, 16);
+				quint16 port = ReadUint16((const Uint8*)arr.data() + i, 16);
 
-				KNetwork::KIpAddress ip(buf,6);
-				addPeer(ip.toString(),ReadUint16(buf,16));
+				addPeer(net::Address(ip, port), false);
 			}
 		}
-		
+
 		delete n;
 		return true;
 	}
-	
+
 	void HTTPTracker::onKIOAnnounceResult(KJob* j)
 	{
 		KIOAnnounceJob* st = (KIOAnnounceJob*)j;
 		KUrl u = st->announceUrl();
-		onAnnounceResult(u,st->replyData(),j);
+		onAnnounceResult(u, st->replyData(), j);
 	}
 
 	void HTTPTracker::onQHttpAnnounceResult(KJob* j)
 	{
 		HTTPAnnounceJob* st = (HTTPAnnounceJob*)j;
 		KUrl u = st->announceUrl();
-		onAnnounceResult(u,st->replyData(),j);
+		onAnnounceResult(u, st->replyData(), j);
 	}
 
-	void HTTPTracker::onAnnounceResult(const KUrl& url,const QByteArray& data,KJob* j)
+	void HTTPTracker::onAnnounceResult(const KUrl& url, const QByteArray& data, KJob* j)
 	{
 		timer.stop();
 		active_job = 0;
@@ -416,8 +423,8 @@ namespace bt
 			error.clear();
 			if (err.isEmpty())
 				err = j->errorString();
-			
-			Out(SYS_TRK|LOG_IMPORTANT) << "Error : " << err << endl;
+
+			Out(SYS_TRK | LOG_IMPORTANT) << "Error : " << err << endl;
 			if (url.queryItem("event") != "stopped")
 			{
 				failures++;
@@ -441,11 +448,11 @@ namespace bt
 						peersReady(this);
 						request_time = QDateTime::currentDateTime();
 						status = TRACKER_OK;
-						requestOK();
 						if (url.queryItem("event") == "started")
 							started = true;
 						if (started)
 							reannounce_timer.start(interval * 1000);
+						requestOK();
 					}
 				}
 				catch (bt::Error & err)
@@ -470,13 +477,13 @@ namespace bt
 		failures++;
 		failed(i18n("Invalid tracker URL"));
 	}
-	
+
 	void HTTPTracker::setupMetaData(KIO::MetaData & md)
 	{
 		md["UserAgent"] = bt::GetVersionString();
 		md["SendLanguageSettings"] = "false";
 		md["cookies"] = "none";
-	//	md["accept"] = "text/plain";
+		//	md["accept"] = "text/plain";
 		md["accept"] = "text/html, image/gif, image/jpeg, *; q=.2, */*; q=.2";
 		if (proxy_on)
 		{
@@ -489,81 +496,81 @@ namespace bt
 				md["UseProxy"] = p;
 			else
 				md["UseProxy"] = QString();
-			
-			Out(SYS_TRK|LOG_DEBUG) << "Using proxy : " << md["UseProxy"] << endl;
+
+			Out(SYS_TRK | LOG_DEBUG) << "Using proxy : " << md["UseProxy"] << endl;
 		}
 	}
-	
+
 	void HTTPTracker::doAnnounceQueue()
 	{
 		if (announce_queue.empty())
 			return;
-		
+
 		KUrl u = announce_queue.front();
 		announce_queue.pop_front();
 		doAnnounce(u);
 	}
-	
+
 	void HTTPTracker::doAnnounce(const KUrl & u)
 	{
-		Out(SYS_TRK|LOG_NOTICE) << "Doing tracker request to url (via " << (use_qhttp ? "QHttp" : "KIO") << "): " << u.prettyUrl() << endl;
-		
+		Out(SYS_TRK | LOG_NOTICE) << "Doing tracker request to url (via " << (use_qhttp ? "QHttp" : "KIO") << "): " << u.prettyUrl() << endl;
+
 		if (!use_qhttp)
 		{
 			KIO::MetaData md;
 			setupMetaData(md);
-			KIOAnnounceJob* j = new KIOAnnounceJob(u,md);
-			connect(j,SIGNAL(result(KJob* )),this,SLOT(onKIOAnnounceResult( KJob* )));
+			KIOAnnounceJob* j = new KIOAnnounceJob(u, md);
+			connect(j, SIGNAL(result(KJob*)), this, SLOT(onKIOAnnounceResult(KJob*)));
 			active_job = j;
 		}
 		else
 		{
 			HTTPAnnounceJob* j = new HTTPAnnounceJob(u);
-			connect(j,SIGNAL(result(KJob* )),this,SLOT(onQHttpAnnounceResult(KJob*)));
+			connect(j, SIGNAL(result(KJob*)), this, SLOT(onQHttpAnnounceResult(KJob*)));
 			if (!proxy_on)
 			{
 				QString proxy = KProtocolManager::proxyForUrl(u); // Use KDE settings
 				if (!proxy.isNull() && proxy != "DIRECT")
 				{
 					KUrl proxy_url(proxy);
-					j->setProxy(proxy_url.host(),proxy_url.port() <= 0 ? 80 : proxy_url.port());
+					j->setProxy(proxy_url.host(), proxy_url.port() <= 0 ? 80 : proxy_url.port());
 				}
 			}
-			else if (!proxy.isNull()) 
+			else if (!proxy.isNull())
 			{
-				j->setProxy(proxy,proxy_port);
+				j->setProxy(proxy, proxy_port);
 			}
 			active_job = j;
 			j->start();
 		}
-		
+
 		timer.start(60*1000);
 		status = TRACKER_ANNOUNCING;
 		requestPending();
 	}
-	
-	void HTTPTracker::onTimeout() 
+
+	void HTTPTracker::onTimeout()
 	{
 		if (active_job)
 		{
-			error = i18n("Timeout contacting tracker %1",url.prettyUrl());
+			error = i18n("Timeout contacting tracker %1", url.prettyUrl());
 			active_job->kill(KJob::EmitResult);
 		}
 	}
 
-	
-	void HTTPTracker::setProxy(const QString & p,const bt::Uint16 port) 
+
+	void HTTPTracker::setProxy(const QString & p, const bt::Uint16 port)
 	{
 		proxy = p;
 		proxy_port = port;
 	}
-	
-	void HTTPTracker::setProxyEnabled(bool on) 
+
+	void HTTPTracker::setProxyEnabled(bool on)
 	{
 		proxy_on = on;
 	}
-	
-	
+
+
 	void HTTPTracker::setUseQHttp(bool on)
 	{
 		use_qhttp = on;
@@ -571,3 +578,6 @@ namespace bt
 
 }
 #include "httptracker.moc"
+
+
+
