@@ -28,10 +28,13 @@ using namespace bt;
 
 namespace utp
 {
-	
-	
-	UnackedPacket::UnackedPacket(const QByteArray& data, bt::Uint16 seq_nr, bt::TimeStamp send_time) 
-		: data(data),seq_nr(seq_nr),send_time(send_time),retransmitted(false)
+
+
+	UnackedPacket::UnackedPacket(const PacketBuffer & packet, bt::Uint16 seq_nr, bt::TimeStamp send_time)
+			: packet(packet),
+			  seq_nr(seq_nr),
+			  send_time(send_time),
+			  retransmitted(false)
 	{
 	}
 
@@ -39,8 +42,13 @@ namespace utp
 	{
 	}
 
-	
-	RemoteWindow::RemoteWindow() : cur_window(0),max_window(64 * 1024),wnd_size(0),last_ack_nr(0),last_ack_receive_count(0)
+
+	RemoteWindow::RemoteWindow()
+			: cur_window(0),
+			  max_window(64 * 1024),
+			  wnd_size(0),
+			  last_ack_nr(0),
+			  last_ack_receive_count(0)
 	{
 
 	}
@@ -50,7 +58,7 @@ namespace utp
 		clear();
 	}
 
-	void RemoteWindow::packetReceived(const utp::Header* hdr,const SelectiveAck* sack,Retransmitter* conn)
+	void RemoteWindow::packetReceived(const utp::Header* hdr, const SelectiveAck* sack, Retransmitter* conn)
 	{
 		if (hdr->ack_nr == last_ack_nr)
 		{
@@ -62,29 +70,26 @@ namespace utp
 			last_ack_nr = hdr->ack_nr;
 			last_ack_receive_count = 1;
 		}
-		
+
 		wnd_size = hdr->wnd_size;
-		
+
 		bt::TimeStamp now = bt::Now();
-		QList<UnackedPacket*>::iterator i = unacked_packets.begin();
+		QList<UnackedPacket>::iterator i = unacked_packets.begin();
 		while (i != unacked_packets.end())
 		{
-			UnackedPacket* up = *i;
-			if (SeqNrCmpSE(up->seq_nr,hdr->ack_nr))
+			if (SeqNrCmpSE(i->seq_nr, hdr->ack_nr))
 			{
 				// everything up until the ack_nr in the header is acked
-				conn->updateRTT(hdr,now - up->send_time,up->data.size());
-				cur_window -= up->data.size();
-				delete up;
+				conn->updateRTT(hdr, now - i->send_time, i->packet.payloadSize());
+				cur_window -= i->packet.payloadSize();
 				i = unacked_packets.erase(i);
 			}
 			else if (sack)
 			{
-				if (Acked(sack,up->seq_nr - hdr->ack_nr))
+				if (Acked(sack, i->seq_nr - hdr->ack_nr))
 				{
-					conn->updateRTT(hdr,now - up->send_time,up->data.size());
-					cur_window -= up->data.size();
-					delete up;
+					conn->updateRTT(hdr, now - i->send_time, i->packet.payloadSize());
+					cur_window -= i->packet.payloadSize();
 					i = unacked_packets.erase(i);
 				}
 				else
@@ -93,61 +98,61 @@ namespace utp
 			else
 				break;
 		}
-		
+
 		if (!unacked_packets.isEmpty())
 		{
-			checkLostPackets(hdr,sack,conn);
+			checkLostPackets(hdr, sack, conn);
 		}
 	}
 
-	void RemoteWindow::addPacket(const QByteArray& data,bt::Uint16 seq_nr,bt::TimeStamp send_time)
+	void RemoteWindow::addPacket(const PacketBuffer & packet, bt::Uint16 seq_nr, bt::TimeStamp send_time)
 	{
-		cur_window += data.size();
-		wnd_size -= data.size();
-		unacked_packets.append(new UnackedPacket(data,seq_nr,send_time));
+		cur_window += packet.payloadSize();
+		wnd_size -= packet.payloadSize();
+		unacked_packets.append(UnackedPacket(packet, seq_nr, send_time));
 	}
 
-	void RemoteWindow::checkLostPackets(const utp::Header* hdr, const utp::SelectiveAck* sack,Retransmitter* conn)
+	void RemoteWindow::checkLostPackets(const utp::Header* hdr, const utp::SelectiveAck* sack, Retransmitter* conn)
 	{
 		bt::TimeStamp now = bt::Now();
 		bool lost_packets = false;
-		
-		QList<UnackedPacket*>::iterator itr = unacked_packets.begin();
-		UnackedPacket* first_unacked = *itr;
-		if (last_ack_receive_count >= 3 && first_unacked->seq_nr == hdr->ack_nr + 1) 
+
+		QList<UnackedPacket>::iterator itr = unacked_packets.begin();
+		UnackedPacket & first_unacked = *itr;
+		if (last_ack_receive_count >= 3 && first_unacked.seq_nr == hdr->ack_nr + 1)
 		{
 			// packet has been lost
-			if (!first_unacked->retransmitted || now - first_unacked->send_time > conn->currentTimeout())
+			if (!first_unacked.retransmitted || now - first_unacked.send_time > conn->currentTimeout())
 			{
 				try
 				{
-					conn->retransmit(first_unacked->data,first_unacked->seq_nr);
-					first_unacked->send_time = now;
-					first_unacked->retransmitted = true;
+					conn->retransmit(first_unacked.packet, first_unacked.seq_nr);
+					first_unacked.send_time = now;
+					first_unacked.retransmitted = true;
 				}
 				catch (utp::Connection::TransmissionError & )
 				{
 				}
 				lost_packets = true;
 			}
-			
+
 			itr++;
 		}
-		
+
 		if (sack)
 		{
 			bt::Uint16 lost_index = lost(sack);
 			while (lost_index > 0 && itr != unacked_packets.end())
 			{
-				bt::Uint16 d = (*itr)->seq_nr - hdr->ack_nr;
-				if (d < lost_index && 
-					(!(*itr)->retransmitted || now - (*itr)->send_time > conn->currentTimeout()))
+				bt::Uint16 d = itr->seq_nr - hdr->ack_nr;
+				if (d < lost_index &&
+				        (!itr->retransmitted || now - itr->send_time > conn->currentTimeout()))
 				{
 					try
 					{
-						conn->retransmit((*itr)->data,(*itr)->seq_nr);
-						(*itr)->send_time = now;
-						(*itr)->retransmitted = true;
+						conn->retransmit(itr->packet, itr->seq_nr);
+						itr->send_time = now;
+						itr->retransmitted = true;
 					}
 					catch (utp::Connection::TransmissionError & )
 					{
@@ -157,10 +162,10 @@ namespace utp
 				itr++;
 			}
 		}
-		
+
 		if (lost_packets)
 		{
-			Out(SYS_UTP|LOG_DEBUG) << "UTP: lost packets on connection " << hdr->connection_id << endl;
+			Out(SYS_UTP | LOG_DEBUG) << "UTP: lost packets on connection " << hdr->connection_id << endl;
 			max_window = (bt::Uint32)qRound(0.78 * max_window);
 		}
 	}
@@ -172,16 +177,16 @@ namespace utp
 		bt::Int16 i = sack->length * 8 - 1;
 		while (i >= 0 && acked < 3)
 		{
-			if (Acked(sack,i))
+			if (Acked(sack, i))
 			{
 				acked++;
 				if (acked == 3)
 					return i;
 			}
-			
+
 			i--;
 		}
-		
+
 		return 0;
 	}
 
@@ -192,13 +197,14 @@ namespace utp
 			max_window = MIN_PACKET_SIZE;
 			bt::TimeStamp now = bt::Now();
 			// When a timeout occurs retransmit packets which are lost longer then the current timeout
-			foreach (UnackedPacket* pkt,unacked_packets)
+			for (QList<UnackedPacket>::iterator i = unacked_packets.begin(); i != unacked_packets.end(); i++)
 			{
-				if (!pkt->retransmitted || now - pkt->send_time > conn->currentTimeout())
+				UnackedPacket & pkt = *i;
+				if (!pkt.retransmitted || now - pkt.send_time > conn->currentTimeout())
 				{
-					conn->retransmit(pkt->data,pkt->seq_nr);
-					pkt->send_time = bt::Now();
-					pkt->retransmitted = true;
+					conn->retransmit(pkt.packet, pkt.seq_nr);
+					pkt.send_time = bt::Now();
+					pkt.retransmitted = true;
 				}
 			}
 		}
@@ -214,14 +220,13 @@ namespace utp
 			max_window = MIN_PACKET_SIZE;
 		else
 			max_window += d;
-		
+
 		//if (scaled_gain > 1000)
 		//	Out(SYS_UTP|LOG_DEBUG) << "RemoteWindow::updateWindowSize " << scaled_gain << " " << max_window << endl;
 	}
-	
+
 	void RemoteWindow::clear()
 	{
-		qDeleteAll(unacked_packets);
 		unacked_packets.clear();
 	}
 }
