@@ -75,7 +75,7 @@ class SendThread : public QThread
 	Q_OBJECT
 public:
 	
-	SendThread(Connection* outgoing,QObject* parent = 0) : QThread(parent),outgoing(outgoing)
+	SendThread(Connection::Ptr outgoing,QObject* parent = 0) : QThread(parent),outgoing(outgoing)
 	{}
 	
 	virtual void run()
@@ -100,7 +100,7 @@ public:
 		outgoing->dumpStats();
 	}
 	
-	Connection* outgoing;
+	Connection::Ptr outgoing;
 };
 
 class CongestionTest : public QEventLoop
@@ -112,9 +112,9 @@ public:
 	}
 	
 public slots:
-	void accepted(Connection* conn)
+	void accepted()
 	{
-		incoming = conn;
+		incoming = srv.acceptedConnection().toStrongRef();
 		exit();
 	}
 	
@@ -150,7 +150,7 @@ private slots:
 	void testConnect()
 	{
 		net::Address addr("127.0.0.1",port);
-		connect(&srv,SIGNAL(accepted(Connection*)),this,SLOT(accepted(Connection*)),Qt::QueuedConnection);
+		connect(&srv,SIGNAL(accepted()),this,SLOT(accepted()),Qt::QueuedConnection);
 		outgoing = srv.connectTo(addr);
 		QVERIFY(outgoing != 0);
 		QTimer::singleShot(5000,this,SLOT(endEventLoop())); // use a 5 second timeout
@@ -205,12 +205,13 @@ private:
 	
 	
 private:
-	Connection* incoming;
-	Connection* outgoing;
+	Connection::Ptr incoming;
+	Connection::Ptr outgoing;
 	CongestionTestServer srv;
 	int port;
 };
 
 QTEST_MAIN(CongestionTest)
 
-#include "congestiontest.moc"
\ No newline at end of file
+#include "congestiontest.moc"
+
